@@ -4,7 +4,7 @@ from pathlib import Path
 from joblib import Parallel, delayed
 
 ## dereplication function to dereplicate a gzipped fasta file
-def dereplication(file, project = None, comp_lvl = None):
+def dereplication(file, project = None, comp_lvl = None, min_size = None):
     """Function to dereplicate a gzipped fasta file. Abundance annotations will be
     written to the output fasta."""
 
@@ -19,8 +19,8 @@ def dereplication(file, project = None, comp_lvl = None):
     ## write stdout to uncompressed output at runtime
     with open(output_path.with_suffix(''), 'w') as output:
         f = subprocess.run(['vsearch',
-                            '--derep_fulllength', Path(file),
-                            '--output', '-', '--quiet', '--fasta_width', str(0),
+                            '--fastx_uniques', Path(file),
+                            '--fastaout', '-', '--quiet', '--fasta_width', str(0),
                             '--log', Path(project).joinpath('6_dereplication_pooling', 'temp', '{}.txt'.format(sample_name_out)),
                             '--sizeout', '--relabel', 'seq:'], stdout = output)
 
@@ -29,6 +29,19 @@ def dereplication(file, project = None, comp_lvl = None):
         shutil.copyfileobj(in_stream, out_stream)
     os.remove(output_path.with_suffix(''))
 
+    ## generate a new output path to stream min_size dereplicated sequences that are used for pooling and clustering
+    ## do so only if min_size if bigger than 1, else use dereplicated files from step before
+    if min_size > 1:
+        output_path = Path(project).joinpath('6_dereplication_pooling', 'data', 'pooling', sample_name_out)
+
+        with open(output_path.with_suffix(''), 'w') as output:
+            f = subprocess.run(['vsearch',
+                                '--fastx_uniques', Path(file),
+                                '--fastaout', '-', '--quiet', '--fasta_width', str(0),
+                                '--log', Path(project).joinpath('6_dereplication_pooling', 'temp', '{}.txt'.format(sample_name_out)),
+                                '--sizeout', '--relabel', 'seq:',
+                                '--minuniquesize', str(min_size)], stdout = output)
+
     ## collect processed and passed reads from the log file
     with open(Path(project).joinpath('6_dereplication_pooling', 'temp', '{}.txt'.format(sample_name_out))) as log_file:
         content = log_file.read().split('\n')
@@ -44,17 +57,28 @@ def dereplication(file, project = None, comp_lvl = None):
         pickle.dump([sample_name_out, finished, version, seqs, unique_seqs], log)
 
 ## function to pool the dereplicated reads, pooled reads are dereplicated again
-def pooling(file_list, project = None, comp_lvl = None):
+def pooling(file_list, project = None, comp_lvl = None, min_size = None):
     """Function to pool the dereplicated reads, needs a path to dereplicated reads folder
     and a project to work in. Both are passed by the main function."""
 
-    ## write the output file
-    with gzip.open(Path(project).joinpath('6_dereplication_pooling', 'data', 'pooling', 'pooled_sequences.fasta.gz'), 'wt', comp_lvl) as pool:
-        number_of_files = len(file_list)
-        for i in range(len(file_list)):
-            with gzip.open(file_list[i], 'rt') as to_pool:
-                shutil.copyfileobj(to_pool, pool)
-            print('{}: Added {} of {} files to the pooled sequences.'.format(datetime.datetime.now().strftime("%H:%M:%S"), i + 1, number_of_files))
+    ## write the output file, depending on the value of min_size, delete files that are only used for pooling
+    ## and no longer needed for mapping
+    if min_size > 1:
+        with gzip.open(Path(project).joinpath('6_dereplication_pooling', 'data', 'pooling', 'pooled_sequences.fasta.gz'), 'wb', comp_lvl) as pool:
+            number_of_files = len(file_list)
+            for i in range(len(file_list)):
+                with open(file_list[i], 'rb') as to_pool:
+                    shutil.copyfileobj(to_pool, pool)
+                os.remove(file_list[i])
+                print('{}: Added {} of {} files to the pooled sequences.'.format(datetime.datetime.now().strftime("%H:%M:%S"), i + 1, number_of_files))
+
+    else:
+        with gzip.open(Path(project).joinpath('6_dereplication_pooling', 'data', 'pooling', 'pooled_sequences.fasta.gz'), 'wb', comp_lvl) as pool:
+            number_of_files = len(file_list)
+            for i in range(len(file_list)):
+                with gzip.open(file_list[i], 'rb') as to_pool:
+                    shutil.copyfileobj(to_pool, pool)
+                print('{}: Added {} of {} files to the pooled sequences.'.format(datetime.datetime.now().strftime("%H:%M:%S"), i + 1, number_of_files))
 
     ## dereplicate the pool with minuniquesize = 2
     print('{}: Dereplicating the pooled sequences for clustering and denoising.'.format(datetime.datetime.now().strftime("%H:%M:%S")))
@@ -64,8 +88,8 @@ def pooling(file_list, project = None, comp_lvl = None):
 
     with open(output_path.with_suffix(''), 'w') as output:
         f = subprocess.run(['vsearch',
-                            '--derep_fulllength', Path(project).joinpath('6_dereplication_pooling', 'data', 'pooling', 'pooled_sequences.fasta.gz'),
-                            '--output', '-', '--quiet', '--fasta_width', str(0),
+                            '--fastx_uniques', Path(project).joinpath('6_dereplication_pooling', 'data', 'pooling', 'pooled_sequences.fasta.gz'),
+                            '--fastaout', '-', '--quiet', '--fasta_width', str(0),
                             '--sizein', '--sizeout',
                             '--minuniquesize', str(2)], stdout = output)
 
@@ -84,6 +108,9 @@ def main(project = Path.cwd()):
     gen_settings = pd.read_excel(Path(project).joinpath('Settings.xlsx'), sheet_name = '0_general_settings')
     cores, comp_lvl = gen_settings['cores to use'].item(), gen_settings['compression level'].item()
 
+    settings = pd.read_excel(Path(project).joinpath('Settings.xlsx'), sheet_name = '6_dereplication_pooling')
+    min_size = settings['min size to pool'].item()
+
     ## collect input files from quality filtering step
     input = glob.glob(str(Path(project).joinpath('5_quality_filtering', 'data', '*.fasta.gz')))
 
@@ -96,7 +123,7 @@ def main(project = Path.cwd()):
         pass
 
     ## parallelize the dereplication
-    Parallel(n_jobs = cores)(delayed(dereplication)(file, project = project, comp_lvl = comp_lvl) for file in input)
+    Parallel(n_jobs = cores)(delayed(dereplication)(file, project = project, comp_lvl = comp_lvl, min_size = min_size) for file in input)
 
     ## write log for the dereplication from pkl logs
     summary_logs = glob.glob(str(Path(project).joinpath('6_dereplication_pooling', 'temp', '*.pkl')))
@@ -118,9 +145,13 @@ def main(project = Path.cwd()):
     wb.save(Path(project).joinpath('Project_report.xlsx'))
     writer.close()
 
-    ## pool the dereplicated files
-    files = glob.glob(str(Path(project).joinpath('6_dereplication_pooling' ,'data', 'dereplication', '*.fasta.gz')))
-    pooling(files, project = project, comp_lvl = comp_lvl)
+    ## pool the dereplicated files, if minsize is set to a value > 1 use those for files for pooling
+    if min_size > 1:
+        files = glob.glob(str(Path(project).joinpath('6_dereplication_pooling' ,'data', 'pooling', '*.fasta')))
+    else:
+        files = glob.glob(str(Path(project).joinpath('6_dereplication_pooling' ,'data', 'dereplication', '*.fasta.gz')))
+
+    pooling(files, project = project, comp_lvl = comp_lvl, min_size = min_size)
 
     ## remove temporary files
     shutil.rmtree(Path(project).joinpath('6_dereplication_pooling', 'temp'))
